@@ -241,7 +241,7 @@ SilcMime silc_mime_decode(SilcMime mime, const unsigned char *data,
       }
       silc_free(line);
 
-      for (i = i; i < data_len; i++) {
+      for ( ; i < data_len; i++) {
 	/* Get boundary data */
 	if (data_len - i >= strlen(b) &&
 	    tmp[i] == '-' && tmp[i + 1] == '-') {
@@ -535,6 +535,9 @@ SilcMime silc_mime_assemble(SilcMimeAssembler assembler, SilcMime partial)
     }
   }
 
+  if (!compbuf)
+    goto err;
+
   /* Now parse the complete MIME message and deliver it */
   complete = silc_mime_decode(NULL, (const unsigned char *)compbuf->head,
 			      silc_buffer_truelen(compbuf));
